'use strict';

var react = require('react');

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return;
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(n);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

var useObserver = function useObserver() {
  var rootElem = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
  var rootMargin = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '0px';
  var threshold = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
  var triggerOnce = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : true;
  var onIsInView = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : function () {};

  var _useState = react.useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      isInView = _useState2[0],
      setIsInView = _useState2[1];

  var observerRef = react.useRef(null);

  var handleCallback = function handleCallback(entries) {
    if (triggerOnce && isInView) return;
    entries.forEach(function (entry) {
      if (entry.isIntersecting) {
        setIsInView(true);
        onIsInView();
      }
    });
  };

  var handleCreateObserver = function handleCreateObserver() {
    var options = {
      root: rootElem,
      rootMargin: rootMargin,
      threshold: threshold
    };
    var observer = new IntersectionObserver(handleCallback, options);
    return observer;
  };

  observerRef.current = react.useMemo(handleCreateObserver, [rootElem, rootMargin, threshold, triggerOnce]);
  react.useEffect(function () {
    return function () {
      observerRef.current && observerRef.current.disconnect();
    };
  });

  var handleObserveElem = function handleObserveElem(elem) {
    if (observerRef.current) observerRef.current.observe(elem);
  };

  var handleGetRef = function handleGetRef(elem) {
    if (elem) handleObserveElem(elem);
  };

  return [isInView, handleGetRef];
};

module.exports = useObserver;
